#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Nov 29 16:54:17 2022

@author: sandro_w
"""
import subprocess
import argparse
import sys
import numpy as np
import geometry

def exec_cmd(cmd):
    p = subprocess.Popen(cmd.split(), stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    strings, err = p.communicate()
    return strings.decode()

def convert_IBRION_6_frequencies_to_yaml(fname, geom="POSCAR"):
    
    geo = geometry.geo(geom,filetype="POSCAR")
    num_atoms = len(geo.coors)
    
    # output the lines with eigenvectors as well
    cmd = "grep THz "+fname+" -A " + str(num_atoms+1)
    rval = exec_cmd(cmd)
    freqs = []
    evecs = []
    for line in rval.split("\n"):
        elements = line.split()
        if len(elements) > 3:
            if elements[1] == "f":
                freqs.append(float(elements[3]))
                evecs.append([])
            elif elements[1] == "f/i=":
                # imaginary frequencies interpreted as negative
                freqs.append(-float(elements[2]))
                evecs.append([])
            elif (len(elements) == 6) & (elements[0] != "X"): # eigenvectors
                evecs[-1].append([float(elements[3]),float(elements[4]),float(elements[5])])
    freqs = np.array(freqs)
    evecs = np.array(evecs)
    slog = freqs.argsort()  # phonopy sorts frequencies the other way around
    freqs = freqs[slog]
    evecs = evecs[slog]
    with open("frequencies.yaml","w") as ofp:
        ofp.write("mesh: [     1,     1,     1 ]\n")
        ofp.write("nqpoint: 1\n")
        ofp.write("natom: %d\n" % num_atoms)
        # here, structure information could be added
        ofp.write("\n")
        ofp.write("phonon:\n")
        ofp.write("- q-position: [    0.0000000,    0.0000000,    0.0000000 ]\n")
        ofp.write("  distance_from_gamma:  0.000000000\n")
        ofp.write("  weight: 1\n")
        ofp.write("  band:\n")
        for fid, freq in enumerate(freqs):
            ofp.write("  - # %d\n" % (fid+1))
            ofp.write("    frequency: %16.10f\n" % freq)
            ofp.write("    eigenvector:\n")
            for aid, evec in enumerate(evecs[fid]):
                ofp.write("    - # atom %d\n" % (aid+1))
                for i in range(3):
                    ofp.write("      - [ %16.10f, %16.10f ]\n" % (evec[i], 0.0))
    return


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="convert frequencies computed with the IBRION=6 functionality in VASP into a phonopy format")


    parser.add_argument("-geom", dest="geom", type=str, default="POSCAR", help="file name of the POSCAR file containing the initial structure")
    parser.add_argument("fname", type=str, default="OUTCAR", help="file name of the VASP output")


    args = parser.parse_args()

    afp = open("p_cmnd.log","a")
    afp.write(" ".join(sys.argv)+"\n")
    afp.close()



    convert_IBRION_6_frequencies_to_yaml(**vars(args))