#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt
import numpy as np
import util
import argparse
import sys

parser = argparse.ArgumentParser(description="get the average lattice parameters from a VASP MD run")

parser.add_argument("-s", dest="tstart", type=int, default=10000, help="time steps to discard at the beginning (equilibration phase)")
parser.add_argument("-noshow", dest="noshow", action="store_true", default=False, help="do not show plots")



args = parser.parse_args()

afp = open("p_cmnd.log","a")
afp.write(" ".join(sys.argv)+"\n")
afp.close()

def read_XDATCAR(fname):
  fp = open(fname)
  cells = []
  for i in range(1):
    fp.readline()
  cellfactor = float(fp.readline())
  cells.append([])
  for i in range(3):
    cells[-1].append(list(map(float,fp.readline().split())))
  fp.readline()  # line with the types of atoms
  atypes = list(map(int,fp.readline().split()))
  natoms = sum(atypes)
  fp.readline()
  lastcoors = []
  for line in fp:
    elements = line.split()
    if len(elements) > 0:
      if (elements[0] != "Direct") & (len(lastcoors) < natoms):
        lastcoors.append(list(map(float,elements)))
        if (len(lastcoors) == natoms):
            cells.append([])
      elif (elements[0] != "Direct") & (len(elements) == 3):
        if len(cells[-1])<3:
          cells[-1].append(list(map(float,elements)))
      elif (elements[0] == "Direct"):
        lastcoors = []
  cells.pop()
  cells = np.array(cells)
  return cells

cells = read_XDATCAR("XDATCAR")

plt.style.use("~/ez-grp_style.mplstyle")

fig,axs = util.align_subplots(9)
timesteps = range(len(cells))
timesteps = np.array(timesteps)
timelog = timesteps > args.tstart
ofp = open("avg_lattice_parameters.dat","w")
labelstrs = ["x","y","z"]
print("mean lattice parameters:")
cellmat_mean = np.zeros((3,3))
for i,subaxs in enumerate(axs):
  for j,ax in enumerate(subaxs):
    plt.sca(ax)
    plt.plot(timesteps,cells[:,i,j])
    meancell = np.mean(cells[args.tstart:,i,j])
    cellmat_mean[i,j] = meancell
    print(labelstrs[i]+labelstrs[j]+" "+str(meancell)+" / $\AA$")
    ofp.write("%12.8f " % (meancell))
    plt.plot([timesteps[0],timesteps[-1]],[meancell,meancell],"k--")
    plt.plot(timesteps[timelog],np.cumsum(cells[:,i,j][timelog])/(timesteps[timelog] - args.tstart),"r-",lw=3)
    
    plt.plot([args.tstart,args.tstart],[min(cells[:,i,j]),max(cells[:,i,j])],"r--",zorder=-5)
    plt.xlabel("time step")
    plt.ylabel(labelstrs[i]+labelstrs[j]+" / $\AA$")
ofp = open("mean_lat_params.dat","w")
for i in range(3):
    print("%12.8f %12.8f %12.8f" % (cellmat_mean[i,0],cellmat_mean[i,1],cellmat_mean[i,2]))
    ofp.write("%12.8f %12.8f %12.8f\n" % (cellmat_mean[i,0],cellmat_mean[i,1],cellmat_mean[i,2]))
for i in range(3):
    print("norm "+str(i+1)+":",np.linalg.norm(cellmat_mean[i]))
plt.tight_layout()
plt.savefig("lattice_parameters.png")

dirstrs = ["a","b","c"]
aglestrs = ["alpha","beta","gamma"]

fig,axs = plt.subplots(3,3,figsize=(16,12))
normcells = np.linalg.norm(cells,axis=2)

angles = [[],[],[]]
volumes = []
for i in range(len(normcells)):
    alpha = np.arccos(np.dot(cells[i,1],cells[i,2])/normcells[i,1]/normcells[i,2])/2/np.pi*360
    beta = np.arccos(np.dot(cells[i,0],cells[i,2])/normcells[i,0]/normcells[i,2])/2/np.pi*360
    gamma = np.arccos(np.dot(cells[i,0],cells[i,1])/normcells[i,0]/normcells[i,1])/2/np.pi*360
    angles[0].append(alpha)
    angles[1].append(beta)
    angles[2].append(gamma)
    volumes.append(np.dot(np.cross(cells[i,0],cells[i,1]),cells[i,2]))
angles = np.array(angles)
volumes = np.array(volumes)
# lengths
for i,ax in enumerate(axs[0]):
    plt.sca(ax)
    plt.plot(timesteps,normcells[:,i])
    meancell = np.mean(normcells[args.tstart:,i])
    cellmat_mean[i] = meancell
    print(dirstrs[i]+" "+str(meancell)+" / $\AA$")
    ofp.write("%12.8f " % (meancell))
    plt.plot([timesteps[0],timesteps[-1]],[meancell,meancell],"k--")
    plt.plot(timesteps[timelog],np.cumsum(normcells[:,i][timelog])/(timesteps[timelog] - args.tstart),"r-",lw=3)
    
    plt.plot([args.tstart,args.tstart],[min(normcells[:,i]),max(normcells[:,i])],"r--",zorder=-5)
    plt.xlabel("time step")
    plt.ylabel(dirstrs[i]+" / $\AA$")

# angles
for i,ax in enumerate(axs[1]):
    plt.sca(ax)
    plt.plot(timesteps,angles[i])
    meancell = np.mean(angles[i,args.tstart:])
    cellmat_mean[i] = meancell
    print(aglestrs[i]+" "+str(meancell)+" / $^{\circ}$")
    ofp.write("%12.8f " % (meancell))
    plt.plot([timesteps[0],timesteps[-1]],[meancell,meancell],"k--")
    plt.plot(timesteps[timelog],np.cumsum(angles[i][timelog])/(timesteps[timelog] - args.tstart),"r-",lw=3)
    
    plt.plot([args.tstart,args.tstart],[min(angles[i]),max(angles[i])],"r--",zorder=-5)
    plt.xlabel("time step")
    plt.ylabel(aglestrs[i]+" / $^{\circ}$")
   
# volume
plt.sca(axs[2,0])
plt.plot(timesteps,volumes)
meancell = np.mean(volumes[args.tstart:])
cellmat_mean[i] = meancell
print("volume "+str(meancell)+" / $\AA^3$")
ofp.write("%12.8f " % (meancell))
plt.plot([timesteps[0],timesteps[-1]],[meancell,meancell],"k--")
plt.plot(timesteps[timelog],np.cumsum(volumes[timelog])/(timesteps[timelog] - args.tstart),"r-",lw=3)

plt.plot([args.tstart,args.tstart],[min(volumes),max(volumes)],"r--",zorder=-5)
plt.xlabel("time step")
plt.ylabel("volume / $\AA^3$")

plt.tight_layout()
plt.savefig("lattice_parameters_abc.png")

if not args.noshow:
    plt.show()
