#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
  This script serves to visualize the local configurations of a VASP ML run as a function of time 
  at the end of the fitting procedure instead of when they were added (which is the difference to plot_local_configurations.py
'''

import matplotlib.pyplot as plt
import subprocess
import sys
import util
import numpy as np

def get_timestep():
    cmd = "grep POTIM OUTCAR -m 1".split()
    p = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    timestep_str, err = p.communicate()
    return float(timestep_str.decode().split()[2])

def read_basis_configurations(fname="ML_ABN"):
    fp = open(fname,"r")
    readnums = False
    currel = None
    nums = []
    confdict = {}
    aiddict = {}
    for line in fp:
        elements = line.split()
        if (elements[0][0] != "-") & (elements[0][0] != "*"):
            if (len(elements) > 3):
                if (elements[0] == "Basis")  & (elements[1] == "set")  & (elements[2] == "for"):
                    readnums = False
                    currel = elements[3]
                    confdict[currel] = []
                    aiddict[currel] = []
            if (len(elements) > 0) & readnums:
                for element in elements:
                    nums.append(int(element))
            elif (len(elements) == 2) & (currel is not None):
                confdict[currel].append(int(elements[0]))
                aiddict[currel].append(int(elements[1]))
                if  len(confdict[currel]) == nums[len(confdict.keys())-1]:
                    currel = None
            if len(elements) > 7:
                if (elements[1] == "numbers")  & (elements[3] == "basis")  & (elements[4] == "sets")  & (elements[7] == "type"):
                    readnums = True
            
    fp.close()
    for key in confdict.keys():
        print("configurations for ",key," found:",len(confdict[key]))
    return confdict, aiddict

def plot_basis_configurations(noshow=False):
    
    # first read the log:
    
    confdict, aiddict = read_basis_configurations()
    
    
    numels = len(confdict.keys())
    
    
    #tstep = np.array(tstep)
    
    #timestep = get_timestep()
    
    pltstyle="~/ez-grp_style.mplstyle"
    plt.style.use(pltstyle)
    
    f,axs = util.align_subplots(numels)
    axs = axs.ravel()
    
    
    for kid,key in enumerate(confdict.keys()):
        plt.sca(axs[kid])
        #LCONFs[key] = np.array(LCONFs[key])
        unique_aids = np.unique(aiddict[key])
        argsrt = np.argsort(confdict[key])
        xvals = np.array(confdict[key])[argsrt]
        at_indices = np.array(aiddict[key])[argsrt]
        total_cumsum = np.zeros((len(xvals)))
        for uaid in unique_aids:
            logical = at_indices == uaid
            cumsum = np.cumsum(logical)
            plt.plot(xvals,cumsum,"k--",alpha=0.5)
            total_cumsum += cumsum
            
        plt.plot(xvals,total_cumsum/len(unique_aids),lw=3)
        
        plt.xlabel("conf. number")
        plt.ylabel("$N_{conf,"+key+"}/N_{atom}$")
        #plt.legend()
    
    plt.tight_layout()
    plt.savefig("configuration_basis.png")
    if not noshow:
        plt.show()


if __name__ == "__main__":
    
    
    afp = open("p_cmnd.log","a")
    afp.write(" ".join(sys.argv)+"\n")
    afp.close()
    
    plot_basis_configurations()
