#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
  This script serves to visualize the local configurations of a VASP ML run as a function of time
'''

import matplotlib.pyplot as plt
import subprocess
import sys
import util
import numpy as np

def get_timestep():
    cmd = "grep POTIM OUTCAR -m 1".split()
    p = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    timestep_str, err = p.communicate()
    return float(timestep_str.decode().split()[2])

def plot_local_configurations(noshow=False):
    
    # first read the log:
    cmd = "grep LCONF ML_LOGFILE".split()
    sp = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    lines, err = sp.communicate()
    lines = lines.decode().split("\n")
    tstep = []
    LCONFs = {}
    for line in lines:
        elements = line.split("#")[0].split()
        if len(elements) > 3:
            tstep.append(int(elements[1]))
            num_entries = int((len(elements)-2)/3)
            for i in range(num_entries):
                elindex = i*3+2
                if elements[elindex] not in LCONFs.keys():
                    LCONFs[elements[elindex]] = []
                LCONFs[elements[elindex]].append([int(elements[elindex+1]),int(elements[elindex+2])])
    
    numels = len(LCONFs.keys())
    tstep = np.array(tstep)
    
    timestep = get_timestep()
    
    pltstyle="~/ez-grp_style.mplstyle"
    plt.style.use(pltstyle)
    
    f,axs = util.align_subplots(numels)
    axs = axs.ravel()
    
    
    for kid,key in enumerate(LCONFs.keys()):
        plt.sca(axs[kid])
        LCONFs[key] = np.array(LCONFs[key])
        plt.plot(tstep*timestep,LCONFs[key][:,0],label="current")
        plt.plot(tstep*timestep,LCONFs[key][:,1],"k--",label="new",alpha=0.5)
        
        plt.xlabel("t / fs")
        plt.ylabel("$N_{conf,"+key+"}$")
        plt.legend()
    
    plt.tight_layout()
    plt.savefig("local_configurations_t.png")
    if not noshow:
        plt.show()
    

if __name__ == "__main__":
    
    
    afp = open("p_cmnd.log","a")
    afp.write(" ".join(sys.argv)+"\n")
    afp.close()
    
    plot_local_configurations()
