#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
 This script is meant to visualize the convergence of an MTP fit based on the 
 console output.
"""

import matplotlib.pyplot as plt
import argparse
import sys
import numpy as np
import os


parser = argparse.ArgumentParser(description="plots the MLIP fit convergence")
parser.add_argument("fname", nargs="*", help="file names of the console output from consecutive fits")

args = parser.parse_args()

afp = open("p_cmnd.log","a")
afp.write(" ".join(sys.argv)+"\n")
afp.close()

pltstyle="~/ez-grp_style.mplstyle"
plt.style.use(pltstyle)

prev_step = 0

print("outputting final values for the cost function")

init_phase = 75
for fn in args.fname:
    with open(fn) as fp:
        cost_fcts = []
        iter_ids = []
        for line in fp:
            elements = line.split()
            if len(elements) > 3:
                if (elements[0] == "BFGS") & (elements[1] == "iter"):
                    cost_fcts.append(float(elements[3].split("=")[1]))
                    iter_ids.append(int(elements[2].split(":")[0]))
        iter_ids = np.array(iter_ids)
        plt.plot(iter_ids+prev_step,cost_fcts)
        print(fn)
        minind = np.argmin(cost_fcts)
        print(cost_fcts[-1],"  (total minimum:",cost_fcts[minind]," at ",minind,")")
        if (minind+1) != len(cost_fcts):
            all_currpath = "/".join(fn.split("/")[:-1])+"/all_curr.mtp"
            if os.path.exists(all_currpath):
                print("found concurrent MTPs in "+all_currpath)
                readind = minind+1 + init_phase
                fp = open(all_currpath)
                cntr = 0
                read_thisone = False
                ofname = "/".join(fn.split("/")[:-1])+"/pot_min.mtp"
                ofp = open(ofname,"w")
                for line in fp:
                    elements = line.split()
                    if len(elements) > 0:
                        if elements[0] == "MTP":
                            cntr +=1
                            if read_thisone:
                                break   # stop reading
                            if cntr == readind:
                                read_thisone = True
                        if read_thisone:
                            ofp.write(line)
                ofp.close()
                print("minimum MTP written in: " + ofname)
        prev_step += iter_ids[-1]
plt.xlabel("iteration")
plt.ylabel("cost function")
plt.yscale("log")
plt.tight_layout()
plt.savefig("MTP_BFGS_convergence.png")
plt.show()
                