#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt
import subprocess
import numpy as np
import argparse
import sys
import copy

def find_nearest(array, value):
    array = np.asarray(array)
    idx = (np.abs(array - value)).argmin()
    return idx

parser = argparse.ArgumentParser(description="plots the MD energy over time from a VASP run")



parser.add_argument("-mean_traj", dest="mean_traj", action="store_true", default=False, help="output the structure closest to the mean energy as MEANEPOSCAR")
parser.add_argument("-mean_traj_mult", dest="mean_traj_mult", type=int, default=1, help="output this many MEANEPOSCARs")

args = parser.parse_args()

plt.style.use("~/ez-grp_style.mplstyle")
afp = open("p_cmnd.log","a")
afp.write(" ".join(sys.argv)+"\n")
afp.close()


cmd = "grep ETOTAL OUTCAR".split()

p = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
strings, err = p.communicate()
strings = strings.decode().split('\n')


cmd = "grep POTIM OUTCAR -m 1".split()
p = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
timestep_str, err = p.communicate()
time_step = float(timestep_str.decode().split()[2])   # time step in fs

energies = []
for line in strings:
  elements = line.split()
  if len(elements) > 3:
    energies.append(float(elements[4]))
      
energies = np.array(energies)
energies_sort = copy.copy(energies)
mean_energy = np.mean(energies)
max_energy = np.max(energies)
if args.mean_traj_mult > 1:
    args.mean_traj = True
if args.mean_traj:
    import swfileIO.dump
    import util
    dumpdata = swfileIO.dump.dump(".",filetype="vasp")
    velocities = dumpdata.compute_velocities()
    print("writing the "+str(args.mean_traj_mult)+" structures closest to the mean energy...")
    print("mean energy: "+str(mean_energy))
    for i in range(args.mean_traj_mult):
        ofname = "MEANEPOSCAR-"+str(i)
        nearest_id = find_nearest(energies_sort,mean_energy)
        print("output structure with id "+str(nearest_id)+" to "+ofname)
        print("structure "+str(nearest_id)+" energy: "+str(energies_sort[nearest_id]))
        cell = dumpdata.cells[nearest_id]
        coors = dumpdata.coors[nearest_id]
        print("sum of velocities:")
        print(np.sum(velocities[nearest_id],axis=0))
        util.write_POSCAR(ofname,cell,dumpdata.elems,coors,comment=None, velocities=velocities[nearest_id])
        energies_sort[nearest_id] = max_energy+1
    

timea = np.array(range(len(energies)))*time_step
plt.plot(timea,energies)
plt.plot([min(timea),max(timea)],[mean_energy,mean_energy],"k--")
plt.xlabel("t / fs")
plt.ylabel("E / eV")
plt.tight_layout()
plt.savefig("mdenergy.png")
plt.show()
