#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt
import subprocess
import numpy as np


def get_md_temp():
    cmd = "grep temperature OUTCAR".split()

    p = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    strings, err = p.communicate()
    strings = strings.decode().split('\n')

    temps = []
    ekins = []
    warned_yet = False
    for line in strings:
        elements = line.split()
        if len(elements) > 2:
            if (elements[0] == "kin.") & (elements[1] == "lattice"):
                if elements[5] == "K)":
                    # in this case the temperature is given as ******* but the temperature could 
                    # still be computed from the kinetic energy with the number of atoms
                    if not warned_yet:
                        print("WARNING: extremely high temperatures")
                        warned_yet = True
                else:
                    temps.append(float(elements[5]))
                ekins.append(float(elements[3]))
        
    temps = np.array(temps)
    ekins = np.array(ekins)
    return temps, ekins

if __name__ == "__main__":
    temps,ekins = get_md_temp()
    
    cmd = "grep POTIM OUTCAR -m 1".split()
    p = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    timestep_str, err = p.communicate()
    time_step = float(timestep_str.decode().split()[2])   # time step in fs




    plt.style.use("~/ez-grp_style.mplstyle")
    timea = np.array(range(len(temps)))*time_step
    plt.plot(timea,temps)
    plt.plot([min(timea),max(timea)],[np.mean(temps),np.mean(temps)],"k--")
    plt.xlabel("t / fs")
    plt.ylabel("T / K")
    plt.tight_layout()
    plt.savefig("mdtemp.png")
    plt.show()
