#!/usr/bin/python3

# this script serves to relax a cell using an mlip potential while using the lammps interface

from ML_FF_aux.mlip_lammps_interface import mlip_lammps_interface 
import geometry
import argparse
import sys
import numpy as np
import swfileIO.ml_abn as ml_abn

from mpi4py import MPI

commw = MPI.COMM_WORLD
rank = commw.Get_rank()



def relax_cell(filen,filetype="POSCAR",potfile="pot.mtp",box=False,maxiter=100,template=None):
   
    outfilename = filen+"_out"

    if template is not None:
        mobj = ml_abn.ml_abn(".",filetype="none")
        mobj.read_MLIP_cfg(template)
        atom_types = np.array(mobj.elems[0])
    else:
        atom_types = None
    geom = geometry.geo(filen, filetype=filetype, atom_types=atom_types)
    if filetype == "lmp":
        geom.masses = geom.type_masses
    geom.convert_cell_to_lammps()
    mli = mlip_lammps_interface()
    mli.load_mlip_potential(potfile,geom)
    e_0 = mli.calc_energy()
    if rank == 0:
        print("total energy of the initial structure:", e_0)
        print("relaxing internal coordinates...")
    if box:
        mli.relax_box(maxiter=maxiter)
    else:
        mli.relax_int(maxiter=maxiter*10) # 10 times as many iterations to be consistent with the number of iteration in a box relaxation
    
    cell = mli.get_cell()
    coors = mli.get_xyz()
    
    ogeom = geometry.geo(outfilename, filetype="direct", elems=geom.elems, coors=coors, cell=cell)
    ogeom.write_file(outfilename, filetype=filetype)
    
    e_end = mli.get_energy()
    if rank == 0:
        print("total energy after relaxation:", e_end)
        print("File written in",outfilename)
    
    


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="relaxes a cell with an mlip potential")


    parser.add_argument("-potfile", dest="potfile", type=str, default="pot.mtp", help="name of the file that stores the FFP, default pot.mtp")
    parser.add_argument("-filetype", dest="filetype", type=str, default="POSCAR", help="which filetype is the geometry file, default POSCAR")
    parser.add_argument("-template", dest="template", type=str, default=None, help="atom type template to apply for the MTP")
    parser.add_argument("-maxiter", dest="maxiter", type=int, default=100, help="maximum number of relaxation steps")
    parser.add_argument("-box", dest="box", action="store_true", default=False, help="perform a box relaxation as well")
    parser.add_argument("filen", type=str, help="file name to be relaxed")


    args = parser.parse_args()
    
    if rank==0:
        afp = open("p_cmnd.log","a")
        afp.write(" ".join(sys.argv)+"\n")
        afp.close()



    relax_cell(**vars(args))
    
