#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import molsys
import numpy as np
from swfileIO.ml_abn import ml_abn
import sys

variant = int(sys.argv[1]) # variant
reffile = sys.argv[2]   # mfpx file
old_cfg_file = sys.argv[3]
new_cfg_file = sys.argv[4]

#fname = "../sposcar"

m = molsys.mol.from_file(reffile)

atypes = np.array(m.get_atypes())
fragtypes = np.array(m.get_fragtypes())
elems = np.array(m.get_elems())




#### VARIANT 1: no restrictions


#### VARIANT 2: O in co2 are the same
if variant == 2:
    atypes[(fragtypes=="co2s") & (elems=="o")] = "o3_c1zn2"

#### VARIANT 3: C atoms in the phenyl ring are the same
elif variant == 3:
    atypes[(fragtypes=="ph") & (elems=="c")] = "c3_c3"

#### VARIANT 4: both VARIANT 2 and 3
elif variant == 4:
    atypes[(fragtypes=="ph") & (elems=="c")] = "c3_c3"
    atypes[(fragtypes=="co2s") & (elems=="o")] = "o3_c1zn2"

full_type_list = []
for aid in range(len(atypes)):
    full_type_list.append(atypes[aid]+"-"+fragtypes[aid])

uatypes = np.unique(full_type_list)
print("unique atom types:",len(uatypes),uatypes)

ml_abn_obj = ml_abn(fname=old_cfg_file,filetype="cfg")


#print(ml_abn_obj.elems)

for cfg_id in range(len(ml_abn_obj.elems)):
    ml_abn_obj.elems[cfg_id] = full_type_list

ml_abn_obj.write_MLIP_cfg(new_cfg_file)
