#!/usr/bin/env python3

from numpy import *
from pylab import *
from subprocess import *
from matplotlib import rc, rcParams
from scipy.optimize import curve_fit
from scipy import stats
import math
import geometry
import os
import sys

from scipy import optimize

def zaoui_sqrt_kappa_func(Lz, kappa_bulk, Lambda):
    return kappa_bulk*(1-sqrt(Lambda/Lz))

afp = open("p_cmnd.log","a")
afp.write(" ".join(sys.argv)+"\n")
afp.close()

sizefont = 22

filen = "sizes.dat"
if len(sys.argv) > 1:
    filen = sys.argv[1]
    
# get lattice parameters from simulation files
# botched only works for standard procedure

#lattice_param = 25.73315 
if len(sys.argv) > 2:
    defaultdir = sys.argv[2]
else:
    defaultdir = "811"
direction = 1
if len(sys.argv) > 3:
    if sys.argv[3][0] != "-":
        direction = int(sys.argv[3])
lparfact = 2
if len(sys.argv) > 4:
    if sys.argv[4][0] != "-":
        lparfact = int(sys.argv[4])

if "-zaoui" in sys.argv:
    use_zaoui = True
else:
    use_zaoui = False
    
cwd = os.getcwd()
os.chdir(defaultdir)
files = os.listdir(".")
datafile = ""
for f in files:
  if (f.split(".")[-1] == "data") | (f.split(".")[-1] == "lmp"):
    if datafile !="":
      print ("WARNING: multiple lammps format data files found, evaluated results could be incorrect")
    datafile = f
geo = geometry.geo(datafile)
lattice_param = geo.cell[direction][direction]/lparfact
print ("Lattice parameter: " + str(lattice_param))

os.chdir(cwd)


data = genfromtxt(filen)
#r0 = 1.43
r0 = 1
cells = 1.0/(data[:,0]*lattice_param)*r0
therm_cond = 1.0/data[:,1]
errors = data[:,2]*therm_cond**2
#Jc = data[:,3]
#Jh = data[:,4]

#fit_all = np.polyfit(cells, therm_cond, 1, full=True)
#fit = fit_all[0]
x = linspace(0,0.0200,100)
if use_zaoui:
    A2m = 1e-10
    guesses = [1000,10*A2m]  # 10 A = 1 nm
    print("performing AEMD square root interpolation")
    params, cov = optimize.curve_fit(zaoui_sqrt_kappa_func, (data[:,0]*lattice_param)*r0*A2m, 
                                     data[:,1], p0=guesses)
    print("covariance:", cov)
    plt.plot(x, 1/zaoui_sqrt_kappa_func(1/x*A2m, params[0], params[1]), label="Zaoui")
    print("kappa_inf:", params[0], "W/mK")
    print("Lambda:", params[1], "m")

print("linear 1st order fit:")
k,d,rvalue,pvalue,stderr = stats.linregress(cells,therm_cond)
fit = [k,d]
fit_fn = np.poly1d(fit) 

intercepterr = stderr*(1.0/len(therm_cond)*sum(asarray(cells)**2))**0.5
student_t = stats.t(len(cells)-2)
confidence = student_t.ppf(0.95)*intercepterr

#subplot(221)
plt.errorbar(cells, therm_cond, yerr = errors, fmt = 'o', capsize=5,markersize=10)
plt.plot(x, fit_fn(x), '--k', label="linear")
print("fit: x*" + str(fit[0]) + " + " + str(fit[1]))
print ("Error of intercept: " + str(intercepterr) + " ; 95% confidence: " + str(confidence))
print("kappa = " + str(1/fit[1]) + " W/m.K stderr: "+ str(intercepterr/fit[1]**2) +" 95% confidence: " + str(confidence/fit[1]**2))
# for easy copying:
print(str(1/fit[1])+" "+str(confidence/fit[1]**2)+" "+str(intercepterr/fit[1]**2))
ofp = open("kappa_infinite.dat","w")
ofp.write(str(1/fit[1])+" "+str(confidence/fit[1]**2)+" "+str(intercepterr/fit[1]**2))
ofp.close()
plt.xlim([0,0.0075])
plt.xlim([0,0.010])
plt.ylim([fit[1]*0.8,max(therm_cond)*1.2])
xlabel(r'1/L / $\mathbf{\AA^{-1}}$',fontsize=sizefont, fontweight="bold")
#xlabel(r'$\mathbf{r_b^{ref}/L}$',fontsize=sizefont, fontweight="bold")
ylabel(r'1/$\mathbf{\kappa}$ / $\mathbf{W^{-1}K m}$',fontsize=sizefont, fontweight="bold")
ax = gca()
plt.tick_params(axis='both', which='major', labelsize=15)
plt.grid()
if use_zaoui:
    plt.legend()
tight_layout()
#show()
savefig("finite_fit.png")


plt.figure(2)


xlen = (data[:,0]*lattice_param)*r0
x = linspace(min(xlen)*0.8,max(xlen)*1.2,100)

plt.plot(x, 1/fit_fn(1/x), '--k', label="linear")

if use_zaoui:
    plt.plot(x, zaoui_sqrt_kappa_func(x*A2m, params[0], params[1]), label="Zaoui")
    
    plt.legend()
    
plt.errorbar(1/cells, 1/therm_cond, yerr = errors, fmt = 'o', capsize=5,markersize=10)

xlabel(r'L / $\mathbf{\AA}$',fontsize=sizefont, fontweight="bold")
#xlabel(r'$\mathbf{r_b^{ref}/L}$',fontsize=sizefont, fontweight="bold")
ylabel(r'$\mathbf{\kappa}$ / $\mathbf{W/(mK)}$',fontsize=sizefont, fontweight="bold")

plt.grid()
plt.tight_layout()
plt.savefig("finite_fit_kappa_L.png")


show()



