#!/usr/bin/env python3
import nemd_result
import os
import argparse
import sys

parser = argparse.ArgumentParser(description="evaluats slab based temperature profile from a NEMD thermal conductivity MD simulation")



parser.add_argument("-ref", dest="ref", type=str, default="", help="file name of reference structure (initial structure). The script will attempt to detect it automatically if not specified")
parser.add_argument("-outfile", dest="outfile", type=str, default="output.lammps", help="file name of the file containing the simulation output to get settings for certain features")
parser.add_argument("-flux_name", dest="flux_name", type=str, default="flux.dat", help="file name of the flux file")
parser.add_argument("-ehex", dest="ehex", action="store_true", default=False, help="do not use flux - use energy from fix ehex instead")
parser.add_argument("-dir", dest="dir", type=int, default=0, help="direction of heat flux, 0 ... x, 1 ... y, 2 ... z")
parser.add_argument("-useflux", dest="useflux", type=int, default=30, help="use this percentage of the flux to end averaging")
parser.add_argument("-fitdists", dest="fitdists", type=float, nargs=4, default=[2.0/16,6.0/16,10.0/16,14.0/16], help="use these cell boundaries for the linear fit of the temperature profile")
parser.add_argument("-printkappas", dest="printkappas", type=int, nargs="*", default=[1,5,10], help="output kappas at these percentage from the temperature profile averaging to the end")
parser.add_argument("-activeflux", dest="activeflux", action="store_true", default=False, help="plot actively averaged flux")
parser.add_argument("-oflux", dest="oflux", action="store_true", default=False, help="use flux from oflux.dat")
parser.add_argument("-lowflux", dest="lowflux", action="store_true", default=False, help="use lower flux instead of average")
parser.add_argument("-rawflux", dest="rawflux", action="store_true", default=False, help="plot raw fluxes instead of average")
parser.add_argument("-units", dest="units", type=str, default="real", help="lammps unit style")
parser.add_argument("-noshow", dest="noshow", action="store_true", default=False, help="do not show the plot")
parser.add_argument("-nolinfitplot", dest="nolinfitplot", action="store_true", default=False, help="do not show the linear fits in the temperature profile")



args = parser.parse_args()


afp = open("p_cmnd.log","a")
afp.write(" ".join(sys.argv)+"\n")
afp.close()

direc = 0

datafile = args.ref

if datafile == "":
  files = os.listdir(".")
  for f in files:
    if f.split(".")[-1] == "data":
      if datafile !="":
        print ("WARNING: multiple lammps format data files found, evaluated results could be incorrect")
      datafile = f

fluxfile = args.flux_name
if args.oflux:
    fluxfile = 'oflux.dat'
if args.ehex:
    fluxfile=""

nr = nemd_result.nemd_result(cfluxname=fluxfile,refname=datafile,profname="tmp_nemd.profile",
                             active_flux=args.activeflux,direc=args.dir,units=args.units,
                             outfile=args.outfile,ehex=args.ehex)
nr.eval_temp_profile(noshow=args.noshow,lowflux=args.lowflux,useflux=args.useflux,lnoise=(not args.rawflux),
                     fitdists=[[args.fitdists[0],args.fitdists[1]],[args.fitdists[2],args.fitdists[3]]],
                     printkappas=args.printkappas,nolinfitplot=args.nolinfitplot)

